<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use Illuminate\Support\Facades\Hash;
use Laravel\Sanctum\PersonalAccessToken;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use App\Http\Resources\LoginResource;
use Exception;

class AuthController extends Controller
{
    public function register(Request $request){
        $request->validate([
            'name' => 'required',
            'email' => 'required|email',
            'password' => 'required|confirmed',
            'tc' => 'required'
        ]);

        if(User::where('email' , $request->email)->first()){
            return response([
                'message' => 'Email Already Exists',
                'status' => 'failed'
            ], 200);
        }

        $user = User::create([
            'name' => $request->name,
            'email' => $request->email,
            'password' => Hash::make($request->password),
            'tc' => json_decode($request->tc)
        ]);

        // Create token
        $tokenResult = $user->createToken($request->email);
        $token = $tokenResult->plainTextToken;

        return response([
            'token' => $token,
            'message' => 'User Created Successfully',
            'status' => 'success'
        ], 200 );
    }

    public function login(Request $request){
        $request->validate([
            'email' => 'required|email',
            'password' => 'required',
        ]);

        $user = User::where('email' , $request->email)->first();
        if($user && Hash::check($request->password , $user->password)){
            $tokenResult = $user->createToken($request->email);
            $token = $tokenResult->plainTextToken;
    
            return response([
                'token' => $token,
                'user' => new LoginResource($user),
                'message' => 'Login Successfully',
                'status' => 'success'
            ], 200);
        }
        else{
            return response([
                'message' => 'The provided credientials are incorrect'
            ]);
        }
    }


    public function logout(){
        $user = auth()->user();
        PersonalAccessToken::where('tokenable_id', $user->id)
            ->where('tokenable_type', get_class($user))
            ->delete();

        return response([
            'message' => 'User logout successfully',
            'status' => 'success'
        ], 200);
    }

    public function loggedUser(){
        $loggeduser = auth()->user();
        return response([
            'users' => $loggeduser,
            'status' => 'success'
        ], 200);
    }


    public function changePassword(Request $request){
        $request->validate([
            'password' => 'required',
        ]);

        $loggeduser = auth()->user();

        if (!$loggeduser) {
            return response([
                'message' => 'User not authenticated',
                'status' => 'failed'
            ], 401);
        }

        $updated = DB::table('users')
            ->where('id', $loggeduser->id)
            ->update(['password' => Hash::make($request->password)]);

        if (!$updated) {
            return response([
                'message' => 'Password change failed',
                'status' => 'failed'
            ], 500);
        }

        return response([
            'message' => 'Password Changed Successfully',
            'status' => 'success'
        ], 200);
    }

    public function registeredUsers()
    {
        try {
            $users = User::all();
            return response()->json([
                'success' => true,
                'data' => $users
            ], 200);
        } catch (Exception $e) {
            Log::error('Error fetching users: ' . $e->getMessage());

            return response()->json([
                'success' => false,
                'message' => 'An error occurred while fetching users.',
                'error' => $e->getMessage()
            ], 500);
        }
    }
}