<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;

use App\Models\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;
use App\Models\User;
use Illuminate\Support\Facades\Hash;
use Illuminate\Mail\Message;
use Illuminate\Support\Str;
use Carbon\Carbon;

class PasswordResetController extends Controller
{
    public function sendresetpasswordEmail(Request $request){
        $request->validate([
            'email' => 'required|email'
        ]);

        // Check if user email exists
        $user = User::where('email', $request->email)->first();
        if (!$user) {
            return response([
                'message' => 'User does not exist'
            ], 404);
        }

        // Token generation
        $token = Str::random(60);

        PasswordReset::create([
            'email' => $request->email,
            'token' => $token,
            'created_at' => Carbon::now()
        ]);

        // Debugging purpose, remove this line in production
        // dump("https://127.0.0.1:8000/api/user/reset/" . $token);


        Mail::send('reset', ['token' => $token], function (Message $message) use ($request, $token) {
            $message->to($request->email);
        });


        return response([
            'message' => 'Password reset email sent. Please check your email.',
            'status' => 'success'
        ], 200);
    }

    public function reset(Request $request , $token){
        //Delete Token older than one minute
        $formatted = Carbon::now()->subMinutes(10)->toDateTimeString();
        PasswordReset::where('created_at', '<=' ,$formatted)->delete();

        $request->validate([
            'password' => 'required|confirmed'
        ]);

        $passwordreset = PasswordReset::where('token' , $token)->first();
        if(!$passwordreset){
            return response([
                'message' => 'Token is invalid or expired',
                'status' => 'failed'
            ]);
        }

        $user = User::where('email', $passwordreset->email)->first();
        $user->password = Hash::make($request->password);
        $user->save();

        //Delete the token after reseting password
        PasswordReset::where('email' , $user->email)->delete();

        return response([
            'message' => 'Password reset successfully',
            'status' => 'success'
        ], 200);

    }
}

